import torch
import numpy as np
import torch.nn as nn
import torch.optim as optim
import matplotlib.pyplot as plt
from IPython import display
display.set_matplotlib_formats('svg')
import torch.nn.functional as F

# https://arxiv.org/abs/1409.4842v1

# the module in pytorch is slightly different
# https://github.com/pytorch/vision/blob/e61538cba036c42bab23ce8f9d205da9889977ae/torchvision/models/inception.py#L200-L235

# https://www.image-net.org/challenges/LSVRC/2014/results.php

# BasicBlock(192, 64, 96, 128, 16, 32, 32)
class MyInceptionBasicBlock(nn.Module):

    def __init__(self, in_c, out_1x1, red_3x3, out_3x3, red_5x5, out_5x5, out_pool_3x3):
        super().__init__()
        
        self.downsample = False
        
        self.path1 = nn.Sequential(
        
                        nn.Conv2d(
                        in_channels=in_c, 
                        out_channels=out_1x1, 
                        kernel_size=(1,1), 
                        stride=1,
                        bias=False)        
                )
        

        #the same input as the path1              
        self.path2 = nn.Sequential()

        #the same input as the path1 and path2              
        self.path3 = nn.Sequential()

        #pooling does not reduce the number of channels so in_c goes to Conv2d again            
        self.path4 = nn.Sequential()

        
    def forward(self, x):        
        path1_out = self.path1(x)
        path2_out = self.path2(x)
        path3_out = self.path3(x)
        path4_out = self.path4(x)
        final_out = torch.cat([path1_out, path2_out, path3_out, path4_out], dim=1)
        return final_out
        

test_tensor = torch.randn(1, 192, 28, 28)
model = MyInceptionBasicBlock(192, 64, 96, 128, 16, 32, 32)

from torchsummary import summary
model = model.to("cpu")
summary(model, (192, 28, 28))

#from torchview import draw_graph
#architecture = 'MyInceptionBasicBlock'
#model_graph = draw_graph(model, input_size=(1,192,28,28), graph_dir ='LR' , roll=True, expand_nested=True, graph_name=f'{architecture}',save_graph=True,filename=f'{architecture}')
#model_graph.visual_graph
